@@ -805,7 +805,7 @@ split_formula_random_gamm4 <- function(formula) {
 }
 
 # utility to recover the full gam + random formula from a stan_gamm4 model
-formula.gamm4 <- function(x) {
+formula.gamm4 <- function(x, ...) {
   formula <- x$formula
   if (is.null(x$glmod)) {
     return(formula)
