@@ -24,16 +24,14 @@ test_that("as.matrix.projection() works", {
     solterms <- args_prj[[tstsetup]]$solution_terms
     ndr_ncl <- ndr_ncl_dtls(args_prj[[tstsetup]])
 
-    # Expected warning (more precisely: regexp which is matched against the
-    # warning; NA means no warning) for as.matrix.projection():
+    m <- as.matrix(prjs[[tstsetup]],
+                   allow_nonconst_wdraws_prj = ndr_ncl$clust_used)
     if (ndr_ncl$clust_used) {
-      # Clustered projection, so we expect a warning:
-      warn_prjmat_expect <- "the clusters might have different weights"
+      wdr_crr <- prjs[[tstsetup]][["wdraws_prj"]]
     } else {
-      warn_prjmat_expect <- NA
+      wdr_crr <- NULL
     }
-    expect_warning(m <- as.matrix(prjs[[tstsetup]]),
-                   warn_prjmat_expect, info = tstsetup)
+    expect_identical(attr(m, "wdraws_prj"), wdr_crr, info = tstsetup)
 
     if (fam_crr == "gauss" || prj_crr == "latent") {
       npars_fam <- "sigma"
@@ -76,22 +74,18 @@ test_that("as.matrix.projection() works", {
         paste0("xca.", xca_idx, "lvl", seq_len(nlvl_fix[xca_idx])[-1])
       )
     }
-    poly_trms <- grep("poly\\(.*\\)", colnms_prjmat_expect, value = TRUE)
-    if (length(poly_trms) > 0) {
-      poly_degree <- sub(".*(poly\\(.*\\)).*", "\\1", poly_trms)
-      if (length(unique(poly_degree)) != 1) {
-        stop("This test needs to be adapted. Info: ", tstsetup)
-      }
-      poly_degree <- unique(poly_degree)
-      poly_degree <- sub(".*,[[:blank:]]+(.*)\\)", "\\1", poly_degree)
-      poly_degree <- as.integer(poly_degree)
+    I_logic_trms <- grep("I\\(.*as\\.logical\\(.*\\)\\)", colnms_prjmat_expect,
+                         value = TRUE)
+    if (length(I_logic_trms)) {
       colnms_prjmat_expect <- c(
-        setdiff(colnms_prjmat_expect, poly_trms),
-        unlist(lapply(poly_trms, function(poly_trms_i) {
-          paste0(poly_trms_i, seq_len(poly_degree))
+        setdiff(colnms_prjmat_expect, I_logic_trms),
+        unlist(lapply(I_logic_trms, function(I_logic_trms_i) {
+          paste0(I_logic_trms_i, "TRUE")
         }))
       )
     }
+    colnms_prjmat_expect <- expand_poly(colnms_prjmat_expect,
+                                        info_str = tstsetup)
     if (pkg_crr == "brms") {
       if (fam_crr == "categ") {
         # Note: Here, we could also derive `yunq_norefcat` from
@@ -107,7 +101,7 @@ test_that("as.matrix.projection() works", {
       }
       colnms_prjmat_expect <- paste0("b_", colnms_prjmat_expect)
     }
-    if ("(1 | z.1)" %in% solterms) {
+    if (any(c("(1 | z.1)", "(xco.1 | z.1)") %in% solterms)) {
       if (pkg_crr == "brms") {
         mlvl_icpt_str <- "Intercept"
         if (fam_crr == "categ") {
@@ -167,8 +161,7 @@ test_that("as.matrix.projection() works", {
           )
         }
       }
-    }
-    if (all(c("(1 | z.1)", "(xco.1 | z.1)") %in% solterms)) {
+      # Correlation:
       if (pkg_crr == "brms") {
         mlvl_icpt_xco_str <- combn(c(mlvl_icpt_str, mlvl_xco_str), m = 2,
                                    FUN = paste, collapse = "__")
@@ -182,7 +175,7 @@ test_that("as.matrix.projection() works", {
     s_nms <- sub("\\)$", "",
                  sub("^s\\(", "",
                      grep("^s\\(.*\\)$", solterms, value = TRUE)))
-    if (length(s_nms) > 0) {
+    if (length(s_nms)) {
       stopifnot(inherits(refmods[[tstsetup_ref]]$fit, "stanreg"))
       # Get the number of basis coefficients:
       s_info <- refmods[[tstsetup_ref]]$fit$jam$smooth
@@ -239,6 +232,76 @@ test_that("as.matrix.projection() works", {
   }
 })
 
+test_that("`allow_nonconst_wdraws_prj = FALSE` causes an error", {
+  skip_if_not(run_prj)
+  for (tstsetup in grep("\\.clust", names(prjs), value = TRUE)) {
+    if (grepl("\\.clust1", tstsetup)) {
+      err_expected <- NA
+    } else {
+      err_expected <- "different .* weights"
+    }
+    expect_error(as.matrix(prjs[[tstsetup]]), err_expected, info = tstsetup)
+  }
+})
+
+test_that(paste(
+  "as_draws_matrix.projection() is a conversion of as.matrix.projection(),",
+  "with different weights of the projected draws causing the application of",
+  "posterior::weight_draws()"
+), {
+  skip_if_not(run_prj)
+  skip_if_not_installed("posterior")
+  for (tstsetup in names(prjs)) {
+    if (args_prj[[tstsetup]]$mod_nm == "gamm") {
+      # Skipping GAMMs because of issue #131.
+      # TODO (GAMMs): Fix this.
+      next
+    }
+    ndr_ncl <- ndr_ncl_dtls(args_prj[[tstsetup]])
+    m <- as.matrix(prjs[[tstsetup]],
+                   allow_nonconst_wdraws_prj = ndr_ncl$clust_used)
+    m_dr <- posterior::as_draws_matrix(prjs[[tstsetup]])
+    m_dr_repl <- posterior::as_draws_matrix(m)
+    if (ndr_ncl$clust_used) {
+      m_dr_repl <- posterior::weight_draws(m_dr_repl,
+                                           weights = attr(m, "wdraws_prj"))
+    }
+    expect_identical(m_dr, m_dr_repl, info = tstsetup)
+  }
+})
+
+test_that(paste(
+  "as_draws_matrix.projection() passes argument `nm_scheme` to",
+  "as.matrix.projection()"
+), {
+  skip_if_not(run_prj)
+  skip_if_not_installed("posterior")
+  for (tstsetup in grep("rstanarm\\.glm\\.", names(prjs), value = TRUE)) {
+    ndr_ncl <- ndr_ncl_dtls(args_prj[[tstsetup]])
+    m <- as.matrix(prjs[[tstsetup]], nm_scheme = "brms",
+                   allow_nonconst_wdraws_prj = ndr_ncl$clust_used)
+    m_dr <- posterior::as_draws_matrix(prjs[[tstsetup]], nm_scheme = "brms")
+    m_dr_repl <- posterior::as_draws_matrix(m)
+    if (ndr_ncl$clust_used) {
+      m_dr_repl <- posterior::weight_draws(m_dr_repl,
+                                           weights = attr(m, "wdraws_prj"))
+    }
+    expect_identical(m_dr, m_dr_repl, info = tstsetup)
+  }
+})
+
+test_that(paste(
+  "as_draws.projection() is a wrapper for as_draws_matrix.projection()"
+), {
+  skip_if_not(run_prj)
+  skip_if_not_installed("posterior")
+  for (tstsetup in grep("rstanarm\\.glm\\.", names(prjs), value = TRUE)) {
+    m_dr <- posterior::as_draws_matrix(prjs[[tstsetup]], nm_scheme = "brms")
+    m_dr_unspec <- posterior::as_draws(prjs[[tstsetup]], nm_scheme = "brms")
+    expect_identical(m_dr, m_dr_unspec, info = tstsetup)
+  }
+})
+
 if (run_snaps) {
   if (testthat_ed_max2) local_edition(3)
   width_orig <- options(width = 145)
@@ -264,21 +327,13 @@ if (run_snaps) {
       ndr_ncl <- ndr_ncl_dtls(args_prj_vs[[tstsetup]])
       nterms_crr <- args_prj_vs[[tstsetup]]$nterms
 
-      # Expected warning (more precisely: regexp which is matched against the
-      # warning; NA means no warning) for as.matrix.projection():
-      if (ndr_ncl$clust_used) {
-        # Clustered projection, so we expect a warning:
-        warn_prjmat_expect <- "the clusters might have different weights"
-      } else {
-        warn_prjmat_expect <- NA
-      }
       prjs_vs_l <- prjs_vs[[tstsetup]]
       if (length(nterms_crr) <= 1) {
         prjs_vs_l <- list(prjs_vs_l)
       }
       res_vs <- lapply(prjs_vs_l, function(prjs_vs_i) {
-        expect_warning(m <- as.matrix(prjs_vs_i),
-                       warn_prjmat_expect, info = tstsetup)
+        m <- as.matrix(prjs_vs_i,
+                       allow_nonconst_wdraws_prj = ndr_ncl$clust_used)
         expect_snapshot({
           print(tstsetup)
           print(prjs_vs_i$solution_terms)
@@ -310,21 +365,13 @@ if (run_snaps) {
       ndr_ncl <- ndr_ncl_dtls(args_prj_cvvs[[tstsetup]])
       nterms_crr <- args_prj_cvvs[[tstsetup]]$nterms
 
-      # Expected warning (more precisely: regexp which is matched against the
-      # warning; NA means no warning) for as.matrix.projection():
-      if (ndr_ncl$clust_used) {
-        # Clustered projection, so we expect a warning:
-        warn_prjmat_expect <- "the clusters might have different weights"
-      } else {
-        warn_prjmat_expect <- NA
-      }
       prjs_cvvs_l <- prjs_cvvs[[tstsetup]]
       if (length(nterms_crr) <= 1) {
         prjs_cvvs_l <- list(prjs_cvvs_l)
       }
       res_cvvs <- lapply(prjs_cvvs_l, function(prjs_cvvs_i) {
-        expect_warning(m <- as.matrix(prjs_cvvs_i),
-                       warn_prjmat_expect, info = tstsetup)
+        m <- as.matrix(prjs_cvvs_i,
+                       allow_nonconst_wdraws_prj = ndr_ncl$clust_used)
         expect_snapshot({
           print(tstsetup)
           print(prjs_cvvs_i$solution_terms)
