@@ -55,6 +55,8 @@ test_that("t.augmat()", {
 ## `[.augmat`() -----------------------------------------------------------
 
 test_that("`[.augmat`()", {
+  # Subsetting:
+  # Subsetting with selection of all elements:
   expect_identical(augmtst[], augmtst)
   # Subsetting columns:
   expect_identical(augmtst[, 1:2],
@@ -70,25 +72,36 @@ test_that("`[.augmat`()", {
                    structure(head(xtst, nobs_orig_tst * ncat_tst),
                              nobs_orig = nobs_orig_tst,
                              class = "augvec"))
-  # Subsetting rows (in fact, this is not of interest, at least currently):
-  xrow1 <- xtst[nobs_orig_tst * ncat_tst * (seq_len(ndraws_tst) - 1L) + 1L]
-  expect_identical(augmtst[1, , drop = FALSE],
-                   structure(t(xrow1),
-                             nobs_orig = nobs_orig_tst,
+  # Subsetting rows:
+  xrows1 <- xtst[as.vector(t(sapply(
+    nobs_orig_tst * (seq_len(ncat_tst) - 1L) + 1L,
+    function(idx) {
+      nobs_orig_tst * ncat_tst * (seq_len(ndraws_tst) - 1L) + idx
+    }
+  )))]
+  xrows1 <- matrix(xrows1, nrow = ncat_tst)
+  expect_identical(augmtst[nobs_orig_tst * (seq_len(ncat_tst) - 1L) + 1L, ,
+                           drop = FALSE],
+                   structure(xrows1,
+                             nobs_orig = 1L,
                              class = "augmat"))
-  expect_identical(augmtst[1, ],
-                   structure(xrow1,
-                             nobs_orig = nobs_orig_tst,
-                             class = "augvec"))
+  expect_error(augmtst[1, ], "is_wholenumber.+ is not TRUE")
+  expect_error(augmtst[1, , drop = FALSE], "is_wholenumber.+ is not TRUE")
   # Subsetting rows and columns:
-  expect_identical(augmtst[1:2, 1:2],
-                   structure(arrtst[1:2, 1, 1:2],
-                             nobs_orig = nobs_orig_tst,
-                             class = "augmat"))
-  expect_identical(augmtst[1, 1],
-                   structure(head(xtst, 1),
-                             nobs_orig = nobs_orig_tst,
+  expect_identical(
+    augmtst[sort(c(nobs_orig_tst * (seq_len(ncat_tst) - 1L) + 1L,
+                   nobs_orig_tst * (seq_len(ncat_tst) - 1L) + 2L)), 1:2],
+    structure(cbind(as.vector(arrtst[1:2, , 1]), as.vector(arrtst[1:2, , 2])),
+              nobs_orig = 2L,
+              class = "augmat")
+  )
+  expect_identical(augmtst[nobs_orig_tst * (seq_len(ncat_tst) - 1L) + 1L, 1],
+                   structure(xrows1[, 1],
+                             nobs_orig = 1L,
                              class = "augvec"))
+  expect_error(augmtst[1:3, 1:2], "is_wholenumber.+ is not TRUE")
+  expect_error(augmtst[1, 1], "is_wholenumber.+ is not TRUE")
+  expect_error(augmtst[1, 1, drop = FALSE], "is_wholenumber.+ is not TRUE")
 
   # Assigning:
   augmtst[1, 1] <- 0
@@ -119,18 +132,17 @@ test_that("t.augvec()", {
 ## `[.augvec`() -----------------------------------------------------------
 
 test_that("`[.augvec`()", {
+  # Subsetting:
   expect_identical(augvtst[], augvtst)
-  expect_identical(augvtst[1:2],
-                   structure(head(xtst, 2),
-                             nobs_orig = nobs_orig_tst,
-                             class = "augvec"))
-  expect_identical(augvtst[1],
-                   structure(head(xtst, 1),
-                             nobs_orig = nobs_orig_tst,
+  xrows1_1 <- xtst[nobs_orig_tst * (seq_len(ncat_tst) - 1L) + 1L]
+  expect_identical(augvtst[nobs_orig_tst * (seq_len(ncat_tst) - 1L) + 1L],
+                   structure(xrows1_1,
+                             nobs_orig = 1L,
                              class = "augvec"))
+  expect_error(augvtst[1], "is_wholenumber.+ is not TRUE")
   expect_identical(augvtst[integer()],
-                   structure(head(xtst, 0),
-                             nobs_orig = nobs_orig_tst,
+                   structure(integer(),
+                             nobs_orig = 0L,
                              class = "augvec"))
 
   # Assigning:
@@ -217,8 +229,10 @@ test_that(paste(
     tstsetup_trad <- sub("\\.augdat\\.", ".trad_compare.", tstsetup)
     if (!tstsetup_trad %in% names(prjs)) next
 
-    prjmat <- suppressWarnings(as.matrix(prjs[[tstsetup]]))
-    prjmat_trad <- suppressWarnings(as.matrix(prjs[[tstsetup_trad]]))
+    cl_used <- ndr_ncl_dtls(args_prj[[tstsetup]])$clust_used
+    prjmat <- as.matrix(prjs[[tstsetup]], allow_nonconst_wdraws_prj = cl_used)
+    prjmat_trad <- as.matrix(prjs[[tstsetup_trad]],
+                             allow_nonconst_wdraws_prj = cl_used)
 
     tol_coefs <- ifelse(
       args_prj[[tstsetup]]$mod_nm == "glmm" &&
@@ -251,11 +265,11 @@ test_that(paste(
       list(object = refmods[[args_prj_i_trad$tstsetup_ref]], regul = 0),
       excl_nonargs(args_prj_i_trad)
     ))
-    prjmat <- suppressWarnings(as.matrix(prj))
-    prjmat_trad <- suppressWarnings(as.matrix(prj_trad))
+    cl_used <- ndr_ncl_dtls(args_prj_i)$clust_used
+    prjmat <- as.matrix(prj, allow_nonconst_wdraws_prj = cl_used)
+    prjmat_trad <- as.matrix(prj_trad, allow_nonconst_wdraws_prj = cl_used)
 
-    tol_coefs <- ifelse(ndr_ncl_dtls(args_prj_i)$clust_used,
-                        1e-9, 1e-14)
+    tol_coefs <- ifelse(cl_used, 1e-9, 1e-14)
     expect_equal(prjmat, prjmat_trad, tolerance = tol_coefs, info = tstsetup)
   }
 })
@@ -276,9 +290,10 @@ test_that(paste(
     pl <- pls[[tstsetup]]
     expect_length(dim(pl$pred), 3)
     expect_identical(dim(pl$pred)[3], 1L)
-    pl$pred <- matrix(pl$pred,
-                      nrow = dim(pl$pred)[1],
-                      ncol = dim(pl$pred)[2])
+    pl$pred <- structure(matrix(pl$pred,
+                                nrow = dim(pl$pred)[1],
+                                ncol = dim(pl$pred)[2]),
+                         wdraws_prj = attr(pl$pred, "wdraws_prj"))
     dimnames(pl$pred) <- list(NULL, as.character(seq_len(ncol(pl$pred))))
     dimnames(pl$lpd) <- list(NULL, as.character(seq_len(ncol(pl$lpd))))
     pl_trad <- pls[[tstsetup_trad]]
@@ -302,8 +317,7 @@ test_that(paste(
   skip_if_not(run_vs)
   tstsetups <- grep("\\.brnll\\..*\\.augdat\\.", names(vss), value = TRUE)
   for (tstsetup in tstsetups) {
-    tstsetup_trad <- sub("\\.augdat\\.default_meth", ".trad_compare.forward",
-                         tstsetup)
+    tstsetup_trad <- sub("\\.augdat\\.", ".trad_compare.", tstsetup)
     if (!tstsetup_trad %in% names(vss)) next
 
     vs <- vss[[tstsetup]]
@@ -344,8 +358,7 @@ test_that(paste(
   skip_if_not(run_vs)
   tstsetups <- grep("\\.brnll\\..*\\.augdat\\.", names(smmrys_vs), value = TRUE)
   for (tstsetup in tstsetups) {
-    tstsetup_trad <- sub("\\.augdat\\.default_meth", ".trad_compare.forward",
-                         tstsetup)
+    tstsetup_trad <- sub("\\.augdat\\.", ".trad_compare.", tstsetup)
     if (!tstsetup_trad %in% names(smmrys_vs)) next
 
     smmry_vs <- smmrys_vs[[tstsetup]]
@@ -378,8 +391,7 @@ test_that(paste(
   tstsetups <- grep("\\.brnll\\..*\\.augdat_stats\\.", names(smmrys_vs),
                     value = TRUE)
   for (tstsetup in tstsetups) {
-    tstsetup_trad <- sub("\\.augdat\\.default_meth", ".trad_compare.forward",
-                         tstsetup)
+    tstsetup_trad <- sub("\\.augdat\\.", ".trad_compare.", tstsetup)
     tstsetup_trad <- sub("\\.augdat_stats\\.", ".binom_stats.", tstsetup_trad)
     if (!tstsetup_trad %in% names(smmrys_vs)) next
 
@@ -415,8 +427,7 @@ test_that(paste(
   skip_if_not(run_cvvs)
   tstsetups <- grep("\\.brnll\\..*\\.augdat\\.", names(cvvss), value = TRUE)
   for (tstsetup in tstsetups) {
-    tstsetup_trad <- sub("\\.augdat\\.default_meth", ".trad_compare.forward",
-                         tstsetup)
+    tstsetup_trad <- sub("\\.augdat\\.", ".trad_compare.", tstsetup)
     if (!tstsetup_trad %in% names(cvvss)) next
 
     cvvs <- cvvss[[tstsetup]]
@@ -475,8 +486,7 @@ test_that(paste(
   tstsetups <- grep("\\.brnll\\..*\\.augdat\\.", names(smmrys_cvvs),
                     value = TRUE)
   for (tstsetup in tstsetups) {
-    tstsetup_trad <- sub("\\.augdat\\.default_meth", ".trad_compare.forward",
-                         tstsetup)
+    tstsetup_trad <- sub("\\.augdat\\.", ".trad_compare.", tstsetup)
     if (!tstsetup_trad %in% names(smmrys_cvvs)) next
 
     smmry_cvvs <- smmrys_cvvs[[tstsetup]]
@@ -546,8 +556,7 @@ test_that(paste(
   tstsetups <- grep("\\.brnll\\..*\\.augdat_stats\\.", names(smmrys_cvvs),
                     value = TRUE)
   for (tstsetup in tstsetups) {
-    tstsetup_trad <- sub("\\.augdat\\.default_meth", ".trad_compare.forward",
-                         tstsetup)
+    tstsetup_trad <- sub("\\.augdat\\.", ".trad_compare.", tstsetup)
     tstsetup_trad <- sub("\\.augdat_stats\\.", ".binom_stats.", tstsetup_trad)
     if (!tstsetup_trad %in% names(smmrys_cvvs)) next
 
