@@ -47,6 +47,7 @@ test_that("`object` of class \"stanreg\" or \"brmsfit\" works", {
 })
 
 test_that("missing `data` fails", {
+  skip_if_not_installed("rstanarm")
   fit_nodata <- suppressWarnings(rstanarm::stan_glm(
     dat$y_glm_gauss ~ dat$xco.1 + dat$xco.2 + dat$xco.3 +
       dat$xca.1 + dat$xca.2 + offset(dat$offs_col),
@@ -62,6 +63,7 @@ test_that("missing `data` fails", {
 })
 
 test_that("`formula` as a character string fails", {
+  skip_if_not_installed("rstanarm")
   # If `formula` is a character string, rstanarm::stan_glm() is not able to find
   # objects supplied to arguments `weights` or `offset`, at least when using
   # devtools::test():
@@ -75,16 +77,34 @@ test_that("`formula` as a character string fails", {
                "^inherits\\(formula, \"formula\"\\) is not TRUE$")
 })
 
+test_that("reference models lacking an intercept work", {
+  args_fit_i <- args_fit$rstanarm.glm.gauss.stdformul.with_wobs.with_offs
+  skip_if_not(!is.null(args_fit_i))
+  fit_fun_nm <- get_fit_fun_nm(args_fit_i)
+  fit_no_icpt <- suppressWarnings(do.call(
+    get(fit_fun_nm, asNamespace(args_fit_i$pkg_nm)),
+    c(list(formula = update(args_fit_i$formula, . ~ . - 1)),
+      excl_nonargs(args_fit_i, nms_excl_add = "formula"))
+  ))
+  expect_message(
+    refmod_no_icpt <- get_refmodel(fit_no_icpt),
+    "Adding an intercept to `formula`",
+    info = "rstanarm.glm.gauss.stdformul.with_wobs.with_offs"
+  )
+  nms_compare <- c("formula", "div_minimizer", "y", "proj_predfun", "wobs",
+                   "wdraws_ref", "offset", "y_oscale")
+  expect_equal(
+    refmod_no_icpt[nms_compare],
+    refmods$rstanarm.glm.gauss.stdformul.with_wobs.with_offs[nms_compare],
+    tolerance = .Machine$double.eps,
+    info = "rstanarm.glm.gauss.stdformul.with_wobs.with_offs"
+  )
+})
+
 test_that("offsets specified via argument `offset` work", {
   args_fit_i <- args_fit$rstanarm.glm.gauss.stdformul.with_wobs.with_offs
   skip_if_not(!is.null(args_fit_i))
-  fit_fun_nm <- switch(args_fit_i$pkg_nm,
-                       "rstanarm" = switch(args_fit_i$mod_nm,
-                                           "glm" = "stan_glm",
-                                           "glmm" = "stan_glmer",
-                                           "stan_gamm4"),
-                       "brms" = "brm",
-                       stop("Unknown `pkg_nm`."))
+  fit_fun_nm <- get_fit_fun_nm(args_fit_i)
   upd_no_offs <- paste(". ~", sub(" \\+ offset\\(offs_col\\)", "",
                                   as.character(args_fit_i$formula[3])))
   fit_offs_arg <- suppressWarnings(do.call(
@@ -100,12 +120,12 @@ test_that("offsets specified via argument `offset` work", {
     tolerance = 1e-12,
     info = "rstanarm.glm.gauss.stdformul.with_wobs.with_offs"
   )
-  nms_compare <- c("eta", "mu", "mu_offs", "dis", "y", "intercept", "wobs",
-                   "wsample", "offset", "y_oscale")
+  nms_compare <- c("div_minimizer", "eta", "mu", "mu_offs", "dis", "y",
+                   "proj_predfun", "wobs", "wdraws_ref", "offset", "y_oscale")
   expect_equal(
     refmod_offs_arg[nms_compare],
     refmods$rstanarm.glm.gauss.stdformul.with_wobs.with_offs[nms_compare],
-    tolerance = 1e-12,
+    tolerance = .Machine$double.eps,
     info = "rstanarm.glm.gauss.stdformul.with_wobs.with_offs"
   )
 })
@@ -114,6 +134,7 @@ test_that(paste(
   "binomial family with 1-column response and weights which are not all ones",
   "errors"
 ), {
+  skip_if_not_installed("rstanarm")
   dat_prop <- within(dat, {
     ybinprop_glm <- y_glm_binom / wobs_col
   })
@@ -134,6 +155,49 @@ test_that(paste(
                "response must contain numbers of successes")
 })
 
+test_that("function calls in group terms fail", {
+  tstsetup <- "brms.glmm.brnll.stdformul.without_wobs.without_offs"
+  args_fit_i <- args_fit[[tstsetup]]
+  skip_if_not(!is.null(args_fit_i))
+  fit_gr <- fits[[tstsetup]]
+  fit_gr$formula <- update(fit_gr$formula,
+                           . ~ . - (xco.1 | z.1) + (xco.1 | gr(z.1)))
+  expect_error(
+    refmod_gr <- get_refmodel(fit_gr),
+    paste("Function calls on the right-hand side of a group-term `|` character",
+          "are not allowed\\."),
+    info = tstsetup
+  )
+})
+
+test_that("extra arguments in s() or t2() terms fail", {
+  args_fit_i <- args_fit$rstanarm.gam.gauss.spclformul.with_wobs.without_offs
+  skip_if_not(!is.null(args_fit_i))
+  fit_fun_nm <- get_fit_fun_nm(args_fit_i)
+  fit_s <- suppressWarnings(do.call(
+    get(fit_fun_nm, asNamespace(args_fit_i$pkg_nm)),
+    c(list(formula = update(args_fit_i$formula,
+                            . ~ . - s(s.1) + s(s.1, bs = "cr"))),
+      excl_nonargs(args_fit_i, nms_excl_add = "formula"))
+  ))
+  expect_error(
+    refmod_s <- get_refmodel(fit_s),
+    "arguments other than predictors are not allowed",
+    info = paste0("rstanarm.gam.gauss.stdformul.with_wobs.without_offs", "__s")
+  )
+  fit_t2 <- suppressWarnings(do.call(
+    get(fit_fun_nm, asNamespace(args_fit_i$pkg_nm)),
+    c(list(formula = update(args_fit_i$formula,
+                            . ~ . - s(s.1) + t2(s.1, bs = "tp"))),
+      excl_nonargs(args_fit_i, nms_excl_add = "formula"))
+  ))
+  expect_error(
+    refmod_t2 <- get_refmodel(fit_t2),
+    "arguments other than predictors are not allowed",
+    info = paste0("rstanarm.gam.gauss.stdformul.with_wobs.without_offs", "__t2")
+  )
+})
+
 test_that("get_refmodel() is idempotent", {
   for (tstsetup in names(refmods)) {
     expect_identical(get_refmodel(refmods[[tstsetup]]),
@@ -147,11 +211,13 @@ test_that("get_refmodel() is idempotent", {
 context("predict.refmodel()")
 
 test_that("invalid `type` fails", {
+  skip_if_not(length(fits) > 0)
   expect_error(predict(refmods[[1]], dat, type = "zzz"),
                "^type should be one of")
 })
 
 test_that("invalid `ynew` fails", {
+  skip_if_not(length(fits) > 0)
   expect_error(predict(refmods[[1]], dat, ynew = dat),
                "^Argument `ynew` must be a numeric vector\\.$")
 })
@@ -165,6 +231,17 @@ test_that(paste(
     fam_crr <- args_ref[[tstsetup]]$fam_nm
     prj_crr <- args_ref[[tstsetup]]$prj_nm
 
+    if (grepl("\\.with_wobs|\\.binom", tstsetup)) {
+      wobs_crr <- wobs_tst
+    } else {
+      wobs_crr <- NULL
+    }
+    if (grepl("\\.with_offs", tstsetup)) {
+      offs_crr <- offs_tst
+    } else {
+      offs_crr <- NULL
+    }
+
     y_crr <- dat[, paste("y", mod_crr, fam_crr, sep = "_")]
     if (prj_crr == "latent") {
       dat_crr <- dat
@@ -181,14 +258,18 @@ test_that(paste(
     }
 
     # Without `ynew`:
-    predref_resp <- predict(refmods[[tstsetup]], dat, type = "response")
-    predref_link <- predict(refmods[[tstsetup]], dat, type = "link")
+    predref_resp <- predict(refmods[[tstsetup]], dat, weightsnew = wobs_crr,
+                            offsetnew = offs_crr, type = "response")
+    predref_link <- predict(refmods[[tstsetup]], dat, weightsnew = wobs_crr,
+                            offsetnew = offs_crr, type = "link")
 
     # With `ynew`:
-    predref_ynew_resp <- predict(refmods[[tstsetup]], dat, ynew = y_crr,
-                                 type = "response")
-    predref_ynew_link <- predict(refmods[[tstsetup]], dat, ynew = y_crr_link,
-                                 type = "link")
+    predref_ynew_resp <- predict(refmods[[tstsetup]], dat,
+                                 weightsnew = wobs_crr, offsetnew = offs_crr,
+                                 ynew = y_crr, type = "response")
+    predref_ynew_link <- predict(refmods[[tstsetup]], dat,
+                                 weightsnew = wobs_crr, offsetnew = offs_crr,
+                                 ynew = y_crr_link, type = "link")
 
     # Checks without `ynew`:
     if (prj_crr %in% c("latent", "augdat")) {
